#=============================================================================
# Copyright (c) 2021, NVIDIA CORPORATION.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#=============================================================================

include(CMakeFindDependencyMacro)

find_dependency(CUDAToolkit)

find_dependency(Boost REQUIRED COMPONENTS context fiber filesystem system)
find_dependency(ucx)

#=============================================================================
# pkg-config Search for hwloc
#
find_package(PkgConfig)

pkg_check_modules(hwloc IMPORTED_TARGET GLOBAL hwloc)

if(hwloc_FOUND)
  # Add an alias to the imported target
  add_library(hwloc::hwloc ALIAS PkgConfig::hwloc)
endif()

#=============================================================================

find_package(rmm 22.06 QUIET)
find_dependency(rmm)

find_dependency(gflags)

find_package(glog 0.6 QUIET)
find_dependency(glog)

find_dependency(gRPC)

find_package(rxcpp 4.1.1.2 QUIET)
find_dependency(rxcpp)

find_dependency(nlohmann_json)

find_package(prometheus-cpp 1.0.0 QUIET)
find_dependency(prometheus-cpp)

find_package(libcudacxx 1.7.0 QUIET)
find_dependency(libcudacxx)

find_package(pybind11 2.8.1 QUIET)
find_dependency(pybind11)


set(rapids_global_targets Boost::context;Boost::fiber;Boost::hana;Boost::filesystem;Boost::system;ucx;ucx::ucp;ucx::uct;ucx_ucx;ucx::ucx;hwloc::hwloc;rmm::rmm;rmm::Thrust;gflags;glog;glog::glog;gRPC::address_sorting;gRPC::gpr;gRPC::grpc;gRPC::grpc_unsecure;gRPC::grpc++;gRPC::grpc++_alts;gRPC::grpc++_error_details;gRPC::grpc++_reflection;gRPC::grpc++_unsecure;gRPC::grpc_plugin_support;gRPC::grpcpp_channelz;gRPC::upb;gRPC::grpc_cpp_plugin;gRPC::grpc_csharp_plugin;gRPC::grpc_node_plugin;gRPC::grpc_objective_c_plugin;gRPC::grpc_php_plugin;gRPC::grpc_python_plugin;gRPC::grpc_ruby_plugin;rxcpp;rxcpp::rxcpp;nlohmann_json::nlohmann_json;prometheus-cpp;prometheus-cpp::core;libcudacxx::libcudacxx;pybind11;pybind11::pybind11)


foreach(target IN LISTS rapids_global_targets)
  if(TARGET ${target})
    get_target_property(_is_imported ${target} IMPORTED)
    get_target_property(_already_global ${target} IMPORTED_GLOBAL)
    if(_is_imported AND NOT _already_global)
        set_target_properties(${target} PROPERTIES IMPORTED_GLOBAL TRUE)
    endif()
  endif()
endforeach()

unset(rapids_global_targets)
unset(rapids_clear_cpm_cache)
